/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.service;

import java.io.IOException;
import java.lang.invoke.MethodHandles;
import java.util.Collection;

import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.controller.AppInfo;
import org.oransc.rappmanager.exception.ServiceException;
import org.oransc.rappmanager.helm.HelmClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

@Service
public class AppService {
    private final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());

    @Autowired
    private AppStore appStore;

    @Autowired
    private HelmClient helmClient;

    @Autowired
    ApplicationConfig appConfig;

    public Collection<App> getAllApps() {
        // Call Kubernetes api to get the apps- pod, deplyment, statefulset, job?
        // Get the apps list this ms managing.
        return appStore.getAllApp();
    }

    public App getApp(String name, String version) throws ServiceException {
        return appStore.getApp(name, version);
    }

    public App saveApp(AppInfo appInfo, MultipartFile chartFile, MultipartFile overrideFile)
        throws IOException, ServiceException {
        return appStore.saveApp(appInfo, chartFile, overrideFile);
    }

    public void deleteApp(App app) {
        appStore.deleteApp(app);
    }

    public void installApp(App app) throws ServiceException {
        // call the kubernetes api to create objects (helm call)
        // Call the kubernetes api to create namespace
        // Validate the override file
        logger.info("Install app {} {}", app.getName(), app.getVersion());
        helmClient.installApp(app);
    }

    public void uninstallApp(App app) throws ServiceException {
        // call the kubernetes api to delete objects (helm call)
        logger.info("Uninstall app {} {}", app.getName(), app.getVersion());
        helmClient.uninstallApp(app);
    }

}
